/*
 * Machete
 * Version 0.989
 * Last update on 15 August 2015
 * Made using FightingICE version 1.22
 */

import gameInterface.AIInterface;
import simulator.Simulator;
import structs.FrameData;
import structs.GameData;
import structs.Key;
import commandcenter.CommandCenter;
import enumerate.State;
import structs.CharacterData;


public class Machete implements AIInterface {

	private Key inputKey;
	private boolean player;
	private FrameData frameData;
	private CommandCenter cc;
	private Simulator simulator;
	private GameData gd;
	
	private int distance;
	private int energy;
	private CharacterData opp;
	private CharacterData my;
	private boolean isGameJustStarted;
	private int xDifference;
	
	@Override
	public void close() {
		// Nothing to do here
	}

	@Override
	public String getCharacter() {
		// Select the player ZEN as per competition rules
		return CHARACTER_ZEN;
	}

	@Override
	public void getInformation(FrameData frameData) {
		// Load the frame data every time getInformation gets called
		this.frameData = frameData;
	}

	@Override
	public int initialize(GameData arg0, boolean player) {
		// Initialize the global variables at the start of the round
		inputKey = new Key();
		this.player = player;
		frameData = new FrameData();
		cc = new CommandCenter();	
		gd = arg0;
		simulator = gd.getSimulator();
		isGameJustStarted = true;
		xDifference = -300;
		return 0;
	}

	@Override
	public Key input() {
		// The input is set up to the global variable inputKey
		// which is modified in the processing part
		return inputKey;
	}

	@Override
	public void processing() {
		// First we check whether we are at the end of the round
		if(!frameData.getEmptyFlag() && frameData.getRemainingTime()>0){
			// Simulate the delay and look ahead 2 frames. The simulator class exists already in FightingICE
			if (!isGameJustStarted)
				frameData = simulator.simulate(frameData, this.player, null, null, 17);
			else
				isGameJustStarted = false; //if the game just started, no point on simulating
			cc.setFrameData(frameData, player);
			distance = cc.getDistanceX();
			energy = frameData.getMyCharacter(player).getEnergy();
			my = cc.getMyCharacter();
			opp = cc.getEnemyCharacter();
			xDifference = my.x - opp.x;
			
			if (cc.getskillFlag()) {
				// If there is a previous "command" still in execution, then keep doing it
				inputKey = cc.getSkillKey();
			}
			else {
				// We empty the keys and cancel skill just in case
				inputKey.empty(); 
				cc.skillCancel();
				// Following is the brain of the reflex agent. It determines distance to the enemy
				// and the energy of our agent and then it performs an action
				if ((opp.energy >= 300) && ((my.hp - opp.hp) <= 300))
					cc.commandCall("FOR_JUMP _B B B");
					// if the opp has 300 of energy, it is dangerous, so better jump!!
					// if the health difference is high we are dominating so we are fearless :)
				else if (!my.state.equals(State.AIR) && !my.state.equals(State.DOWN)) { //if not in air
					if ((distance > 150)) {
						cc.commandCall("FOR_JUMP"); //If its too far, then jump to get closer fast
					}
					else if (energy >= 300)
						cc.commandCall("STAND_D_DF_FC"); //High energy projectile
					else if ((distance > 100) && (energy >= 50))
						cc.commandCall("STAND_D_DB_BB"); //Perform a slide kick
					else if (opp.state.equals(State.AIR)) //if enemy on Air
						cc.commandCall("STAND_F_D_DFA"); //Perform a big punch
					else if (distance > 100)
						cc.commandCall("6 6 6"); // Perform a quick dash to get closer
					else
						cc.commandCall("B"); //Perform a kick in all other cases, introduces randomness
				}
				else if ((distance <= 150) && (my.state.equals(State.AIR) || my.state.equals(State.DOWN)) 
						&& (((gd.getStageXMax() - my.x)>=200) || (xDifference > 0))
						&& ((my.x >=200) || xDifference < 0)) { //Conditions to handle game corners
					if (energy >= 5) 
						cc.commandCall("AIR_DB"); // Perform air down kick when in air
					else
						cc.commandCall("B"); //Perform a kick in all other cases, introduces randomness
				}	
				else
					cc.commandCall("B"); //Perform a kick in all other cases, introduces randomness
			}
		}
		else isGameJustStarted = true;
	}
}
