package StrategiesClass;

import java.util.Deque;
import java.util.Iterator;
import java.util.LinkedList;

import SkillEnum.AttackSkills;
import enumerate.Action;
import simulator.Simulator;
import structs.FrameData;
import structs.GameData;

public class PredictNextSkill {

	public int myHp;
	public int oppHp;
	public int myEnergy;
	public int oppEnergy;
	public int myX;
	public int myY;
	public int oppX;
	public int oppY;

	GameData gameData;
	boolean playerNumber;
	Simulator simulator;

	Deque<Action> myActionList;
	Deque<Action> oppActionList;

	Deque<Action> myAtk;
	Deque<Action> GroundAtkAct;
	Deque<Action> AirAtkAct;
	Deque<Action> GuardAndEvade;

	Deque<Action> myAct;
	Deque<Action> oppAct;

	public AttackSkills myAS;
	public AttackSkills oppAS;

	public PredictNextSkill(GameData theGameData, boolean thePlayerNumber) {

		myHp = 0;
		oppHp = 0;
		myEnergy = 0;
		oppEnergy = 0;
		myX = 0;
		myY = 0;
		oppX = 0;
		oppY = 0;
		myAS = AttackSkills.STAND_A;
		oppAS = AttackSkills.STAND_A;

		playerNumber = thePlayerNumber;

		this.myAct = new LinkedList<Action>();
		this.oppAct = new LinkedList<Action>();

		initializeSkillDepue();

		gameData = theGameData;
		simulator = gameData.getSimulator();
	}

	public PredictNextSkill(int MyHP, int OppHP, int MyEnergy, int OppEnergy, int MyX, int MyY, int OppX, int OppY,
			AttackSkills MyAS, AttackSkills OppAS) {

		myHp = MyHP;
		oppHp = OppHP;
		myEnergy = MyEnergy;
		oppEnergy = OppEnergy;
		myX = MyX;
		myY = MyY;
		oppX = OppX;
		oppY = OppY;
		myAS = MyAS;
		oppAS = OppAS;

		this.myAct = new LinkedList<Action>();
		this.oppAct = new LinkedList<Action>();

	}

	public void simulateNext(FrameData frameData) {

		myAct.clear();
		myAct.add(Action.STAND_D_DB_BB);

		oppAct.clear();
		oppAct.add(Action.STAND_A);

		FrameData simulatedFrame;

		simulatedFrame = simulator.simulate(frameData, playerNumber, myAct, oppAct, 60);

		simulatedFrame.getMyCharacter(playerNumber).getHp();
		simulatedFrame.getOpponentCharacter(playerNumber).getHp();

		System.out.println(simulatedFrame.getMyCharacter(playerNumber).getHp() + ","
				+ simulatedFrame.getOpponentCharacter(playerNumber).getHp());

	}

	public Action selectNextSkill(FrameData frameData) {

		Deque<Action> myNowTestingAct = new LinkedList<Action>();
		Deque<Action> oppNowTestingAct = new LinkedList<Action>();

		FrameData simulatedFrame;

		int myHP = frameData.getMyCharacter(playerNumber).getHp();
		int oppHP = frameData.getOpponentCharacter(playerNumber).getHp();

		Action nextSkill;
		Action bestSkill = null;

		float damageExpectations = 0;
		float bestSkillExp = 0;

		myActionList = myAtk;
		oppActionList = frameData.getOpponentCharacter(playerNumber).getState().equals(Action.AIR) ? AirAtkAct
				: GroundAtkAct;

		for (Iterator<Action> n = myActionList.iterator(); n.hasNext();) {
			myNowTestingAct.clear();
			nextSkill = n.next();
			myNowTestingAct.add(nextSkill);

			float timesOfHit = 0f;
			float timesOfAll = 0f;

			damageExpectations = 0f;

			for (Iterator<Action> m = oppActionList.iterator(); m.hasNext();) {

				oppNowTestingAct.clear();
				oppNowTestingAct.add(m.next());
				timesOfAll++;
				simulatedFrame = simulator.simulate(frameData, playerNumber, myNowTestingAct, oppNowTestingAct, 60);

				int simMyHP = simulatedFrame.getMyCharacter(playerNumber).getHp();
				int simOppHP = simulatedFrame.getOpponentCharacter(playerNumber).getHp();

				float simScore;
				float nowScore;
				float improvedScore = 0;
				// simScore
				if (simMyHP == 0 && simOppHP == 0) {
					simScore = 0.5f;
				} else if (simMyHP == 0 && simOppHP != 0) {
					simScore = 1f;
				} else if (simMyHP != 0 && simOppHP == 0) {
					simScore = 0f;
				} else {
					simScore = (float) simOppHP / (simMyHP + simOppHP);
				}
				// nowScore
				if (myHP == 0 && oppHP == 0) {
					nowScore = 0.5f;
				} else if (myHP == 0 && oppHP != 0) {
					nowScore = 1f;
				} else if (myHP != 0 && oppHP == 0) {
					nowScore = 0f;
				} else {
					nowScore = (float) oppHP / (myHP + oppHP);
				}

				improvedScore = simScore - nowScore;

				if (simMyHP < myHP && simOppHP < oppHP) {
					// We can hit each other.
					timesOfHit++;
					damageExpectations += improvedScore;
				} else if (simMyHP == myHP && simOppHP < oppHP) {
					// I can hit him,he can't hit me.
					timesOfHit++;
					damageExpectations += improvedScore;
				} else if (simMyHP < myHP && simOppHP == oppHP) {
					// I can't hit him,he can hit me.

				} else if (simMyHP == myHP && simOppHP == oppHP) {
					// We can't hit each other.

				}

			}

			damageExpectations = damageExpectations / timesOfAll * timesOfHit / timesOfAll;

			if (damageExpectations >= bestSkillExp) {
				bestSkillExp = damageExpectations;
				bestSkill = nextSkill;
			}
		}

		return bestSkill;

	}

	public void initializeSkillDepue() {

		this.myActionList = new LinkedList<Action>();
		this.oppActionList = new LinkedList<Action>();

		this.GroundAtkAct = new LinkedList<Action>();
		this.AirAtkAct = new LinkedList<Action>();

		this.myAtk = new LinkedList<Action>();

		this.GuardAndEvade = new LinkedList<Action>();

		myAtk.add(Action.CROUCH_FA);
		myAtk.add(Action.CROUCH_FB);
		myAtk.add(Action.STAND_FA);
		myAtk.add(Action.CROUCH_A);
		myAtk.add(Action.CROUCH_B);
		myAtk.add(Action.STAND_A);
		myAtk.add(Action.STAND_B);
		myAtk.add(Action.THROW_B);
		myAtk.add(Action.THROW_A);

		myAtk.add(Action.STAND_D_DF_FA);
		myAtk.add(Action.STAND_D_DF_FB);
		myAtk.add(Action.STAND_F_D_DFA);
		myAtk.add(Action.STAND_F_D_DFB);
		myAtk.add(Action.STAND_D_DB_BA);
		myAtk.add(Action.STAND_D_DB_BB);
		myAtk.add(Action.STAND_D_DF_FC);

		myAtk.add(Action.AIR_A);
		myAtk.add(Action.AIR_DA);
		myAtk.add(Action.AIR_FA);
		myAtk.add(Action.AIR_UA);
		myAtk.add(Action.AIR_D_DF_FA);
		myAtk.add(Action.AIR_F_D_DFA);
		myAtk.add(Action.AIR_D_DB_BA);

		GroundAtkAct.add(Action.CROUCH_FA);
		GroundAtkAct.add(Action.CROUCH_FB);
		GroundAtkAct.add(Action.STAND_FA);
		GroundAtkAct.add(Action.CROUCH_A);
		GroundAtkAct.add(Action.CROUCH_B);
		GroundAtkAct.add(Action.STAND_A);
		GroundAtkAct.add(Action.STAND_B);
		GroundAtkAct.add(Action.THROW_B);
		GroundAtkAct.add(Action.THROW_A);

		GroundAtkAct.add(Action.STAND_D_DF_FA);
		GroundAtkAct.add(Action.STAND_D_DF_FB);
		GroundAtkAct.add(Action.STAND_F_D_DFA);
		GroundAtkAct.add(Action.STAND_F_D_DFB);
		GroundAtkAct.add(Action.STAND_D_DB_BA);
		GroundAtkAct.add(Action.STAND_D_DB_BB);

		AirAtkAct.add(Action.AIR_A);
		AirAtkAct.add(Action.AIR_DA);
		AirAtkAct.add(Action.AIR_FA);
		AirAtkAct.add(Action.AIR_UA);
		AirAtkAct.add(Action.AIR_D_DF_FA);
		AirAtkAct.add(Action.AIR_F_D_DFA);
		AirAtkAct.add(Action.AIR_D_DB_BA);

	}

}
